@@ -1,6 +1,8 @@
 #!/usr/bin/env python
 # encoding: utf-8
 
+#    babtools_gentoo - Simple tools for Gentoo users and ebuild dabblers. 
+# 
 #    Copyright © 2008 Arne Babenhauserheide
 # 
 #    This program is free software: you can redistribute it and/or modify
@@ -16,7 +18,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>
 
-"""Some tools for working with Gentoo.
+"""babtools_gentoo: Some tools for working with Gentoo.
  
 
 Usage: 
@@ -37,6 +39,13 @@ Avaible commands:
  - ebuild_download_to_local CP URL
   
    Download an ebuild from bugs.gentoo.org to the local portage tree.    
+   
+ - emerge_from_pypi PACKAGE
+  
+   Create an ebuild from a project in PyPI, put it into package.keywords and install it. 
+   Honors package.keywords directories. 
+   It only puts the directly named PACKAGE into package.keywords as dev-python/PACKAGE
+   but doesn't put the dependencies into package.keywords. 
  
 
 Examples: 
@@ -44,9 +53,13 @@ Examples:
    
    copy dev-lang/python into the local overlay
    
- - sudo python babtools_gentoo.py ebuild_download_to_local games-rpg/vegastrike http://bugs.gentoo.org/attachment.cgi?id=151789
+ - sudo babtools_gentoo.py ebuild_download_to_local games-rpg/vegastrike http://bugs.gentoo.org/attachment.cgi?id=151789
    
    Download the ebuild for games-rpg/vegastrike from bugs.gentoo.org and digest it. 
+  
+ - sudo babtools_gentoo.py emerge_from_pypi magma
+   
+   Create an ebuild for "magma" from the PyPI and emerge it. 
 
 
 Source URL (Mercurial): U{http://freehg.org/u/ArneBab/babtools_gentoo/}
@@ -55,7 +68,7 @@ PyPI URL: U{http://pypi.python.org/pypi/babtools_gentoo}
 """
 
 # We need to be able to join paths and to check, if something's a dir or a file. 
-from os.path import join, isdir, isfile
+from os.path import join, isdir, isfile, join
 
 # And to creade directories, and list the contents of a directory. 
 from os import makedirs, listdir
@@ -81,6 +94,8 @@ except:
 
 verbose = True
 
+PACKAGE_KEYWORDS = join(join("/etc", "portage"), "package.keywords")
+
 def copy_ebuild_to_local_overlay(CP):  
     """Copy the ebuild dir to the local overlay.
     
@@ -175,7 +190,75 @@ def create_local_ebuild_dir(CP):
     # If the local dir doesn't exists, create it. 
     if not isdir(EBUILD_DIR_LOCAL): 
         makedirs(EBUILD_DIR_LOCAL)
+
+
+def add_to_package_keywords(CP):
+    """Add a cathegory/package to package.keywords. Honor package.keywords directories. """
+    # First we assemble the CP string to write to the keywords. 
+    string_to_write = "\n# Package added by babtools_gentoo"
+    string_to_write += "\n" + CP
+        
+    # First we check, if package.keywords is a directory. 
+    if isdir(PACKAGE_KEYWORDS): 
+        # If package.keywords is a directory, we check, if CP is inside the file python inside it. 
+        python_keywords_file = open(join(PACKAGE_KEYWORDS, "python"), "r")
+        if not CP in python_keywords_file.read(): 
+            # If CP isn't in the file, 
+            # Remember to close the file first. 
+            python_keywords_file.close()
+            # Then append the CP to the file. 
+            python_keywords_file = open(join(PACKAGE_KEYWORDS, "python"), "a")
+            python_keywords_file.write(string_to_write)
+            # And remember to close the file again. 
+        else: 
+            print CP + " is already inside the package.keywords"
+        python_keywords_file.close()
+    else: 
+        # If package.keywords is a file, we just append the CP to package.keywords, if it isn't already inside it. 
+        keywords_file = open(PACKAGE_KEYWORDS, "r")
+        if not CP in keywords_file.read(): 
+            # First close the file properly. 
+            keywords_file.close()
+            # Then add the CP. 
+            keywords_file = open(PACKAGE_KEYWORDS, "a")
+            keywords_file.write(string_to_write)
+        else: 
+            print CP + " is already inside the package.keywords"
+        # And remember to close it. 
+        keywords_file.close()
+        
+
+def emerge_from_pypi(PACKAGE):
+    """Create an ebuild directly from PyPI.
     
+        - Create the ebuild with g-pypi
+        - put it into package.keywords (if it's a dir, put it into the file python inside the dir)
+        - emerge it via portage. 
+        """
+    # First we need to be able to call shell commands. 
+    from subprocess import call
+    
+    # We now create CP from the PACKAGE
+    CP = "dev-python/" + PACKAGE
+    
+    # Now we call g-pypi to create the ebuild. 
+    if verbose: 
+        print "Creating ebuild for " + CP + " via g-pypi."
+    call("g-pypi " + PACKAGE, shell=True)
+    
+    if verbose: 
+        print "\nPutting " + CP + " into the package.keywords, if it isn't already there."
+    # Then we put dev-python/PACKAGE into package.keywords
+    add_to_package_keywords(CP)
+    
+    if verbose: 
+        print "\nFinally emerging " + CP + "."
+    # And emerge it via a shellcall to portage/emerge. 
+    call("emerge " + CP, shell=True)
+    
+    # At the end, say what we did. 
+    print "\nFinished installing " + CP + "."
+
 
 def print_help(): 
     """Display help and a list of commands."""
@@ -234,6 +317,14 @@ if __name__  == "__main__":
             print_help()
         else: 
             download_ebuild_to_local_overlay(CP=argv[2], URL=argv[3])
+    
+    # If we get the request to directly install a package from the PyPI, then get it. 
+    
+    elif argv[1] == "emerge_from_pypi": 
+        if len(argv) < 3: 
+            print_help()
+        else: 
+            emerge_from_pypi(PACKAGE=argv[2])
     else: 
         print "command not recognized:", argv[1]
         print_help()
